# ./addons/eventscripts/_libs/python/weaponlib.py

import es


class WeaponManager:
   class Weapon:
      def __init__(self, name, ammoprop='', tags=(), slot=0, clip=0, maxammo=0):
         self.info_name     = name
         self.info_ammoprop = ammoprop
         self.info_tags     = (tags,) if isinstance(tags, str) else tuple(tags)
         self.info_slot     = slot
         self.info_clip     = clip
         self.info_maxammo  = maxammo

      def __str__(self):
         return self.get('name')

      def __coerce__(self, x):
         return (self.get('name'), x) if isinstance(x, str) else None

      def __getattr__(self, x):
         try:
            return self.get(x)

         except ValueError:
            raise AttributeError, 'Weapon instance has no attribute \'%s\'' % x

      def __getitem__(self, x):
         try:
            return self.get(x)

         except ValueError:
            raise KeyError, '\'%s\'' % x

      """ Public functions """

      def get(self, info):
         if info == 'name':
            return 'weapon_' + self.info_name

         elif info == 'prop':
            return ('CBasePlayer.localdata.m_iAmmo.' + self.info_ammoprop) if self.info_ammoprop else None

         elif info == 'tags':
            return self.info_tags

         elif info == 'slot':
            return self.info_slot

         elif info == 'clip':
            return self.info_clip

         elif info == 'maxammo':
            return int(es.ServerVar('ammo_' + self.info_maxammo + '_max')) if isinstance(self.info_maxammo, str) else self.info_maxammo

         elif info == 'indexlist':
            return es.createentitylist(self.name).keys()

         raise ValueError, 'No weapon info \'%s\'' % info

   """ Begin WeaponManager """

   def __init__(self):
      self.weapons = {}

   """ Public functions """

   def getWeapon(self, name):
      name = self._formatName(name)

      return self.weapons[name] if name in self.weapons else None

   def getWeaponList(self, tag):
      if str(tag).startswith('#'):
         return filter(lambda x: tag in x.tags, map(lambda x: self.weapons[x], self.weapons))

      else:
         name = self._formatName(tag)
         return [self.weapons[name]] if name in self.weapons else []

   def getWeaponNameList(self, tag):
      return map(str, self.getWeaponList(tag))

   def getIndexList(self, tag):
      indexlist = []
      for weapon in self.getWeaponList(tag):
         indexlist += weapon.indexlist

      return indexlist

   """ Private functions """

   def _registerWeapon(self, name, ammoprop='', tags=(), slot=0, clip=0, maxammo=0):
      name = self._formatName(name)

      self.weapons[name] = self.Weapon(name, ammoprop, tags, slot, clip, maxammo)

   def _formatName(self, name):
      return str(name).lower().replace('weapon_', '')


###

# CS:S weapons

cstrike = WeaponManager()
cstrike._registerWeapon('deagle',       '001', ('#all', '#secondary', '#pistol'),         2, 7,      '50AE')
cstrike._registerWeapon('ak47',         '002', ('#all', '#primary', '#rifle'),            1, 30,     '762mm')
cstrike._registerWeapon('scout',        '002', ('#all', '#primary', '#rifle', '#sniper'), 1, 10,     '762mm')
cstrike._registerWeapon('aug',          '002', ('#all', '#primary', '#rifle'),            1, 30,     '762mm')
cstrike._registerWeapon('g3sg1',        '002', ('#all', '#primary', '#rifle', '#sniper'), 1, 20,     '762mm')
cstrike._registerWeapon('galil',        '003', ('#all', '#primary', '#rifle'),            1, 35,     '556mm')
cstrike._registerWeapon('famas',        '003', ('#all', '#primary', '#rifle'),            1, 25,     '556mm')
cstrike._registerWeapon('m4a1',         '003', ('#all', '#primary', '#rifle'),            1, 30,     '556mm')
cstrike._registerWeapon('sg552',        '003', ('#all', '#primary', '#rifle'),            1, 30,     '556mm')
cstrike._registerWeapon('sg550',        '003', ('#all', '#primary', '#rifle', '#sniper'), 1, 30,     '556mm')
cstrike._registerWeapon('m249',         '004', ('#all', '#primary'),                      1, 100,    '556mm_box')
cstrike._registerWeapon('awp',          '005', ('#all', '#primary', '#rifle', '#sniper'), 1, 10,     '338mag')
cstrike._registerWeapon('tmp',          '006', ('#all', '#primary', '#smg'),              1, 30,     '9mm')
cstrike._registerWeapon('mp5navy',      '006', ('#all', '#primary', '#smg'),              1, 30,     '9mm')
cstrike._registerWeapon('glock',        '006', ('#all', '#secondary', '#pistol'),         2, 20,     '9mm')
cstrike._registerWeapon('elite',        '006', ('#all', '#secondary', '#pistol'),         2, 30,     '9mm')
cstrike._registerWeapon('m3',           '007', ('#all', '#primary', '#shotgun'),          1, 8,      'buckshot')
cstrike._registerWeapon('xm1014',       '007', ('#all', '#primary', '#shotgun'),          1, 7,      'buckshot')
cstrike._registerWeapon('mac10',        '008', ('#all', '#primary', '#smg'),              1, 30,     '45acp')
cstrike._registerWeapon('ump45',        '008', ('#all', '#primary', '#smg'),              1, 25,     '45acp')
cstrike._registerWeapon('usp',          '008', ('#all', '#secondary', '#pistol'),         2, 12,     '45acp')
cstrike._registerWeapon('p228',         '009', ('#all', '#secondary', '#pistol'),         2, 13,     '357sig')
cstrike._registerWeapon('fiveseven',    '010', ('#all', '#secondary', '#pistol'),         2, 20,     '57mm')
cstrike._registerWeapon('p90',          '010', ('#all', '#primary', '#smg'),              1, 50,     '57mm')
cstrike._registerWeapon('hegrenade',    '011', ('#all', '#grenade'),                      4, maxammo='hegrenade')
cstrike._registerWeapon('flashbang',    '012', ('#all', '#grenade'),                      4, maxammo='flashbang')
cstrike._registerWeapon('smokegrenade', '013', ('#all', '#grenade'),                      4, maxammo='smokegrenade')
cstrike._registerWeapon('knife',          tags=('#all', '#knife'),                   slot=3)
cstrike._registerWeapon('c4',             tags=('#all', '#objective'),               slot=5)


###

# HL2DM weapons

hl2mp = WeaponManager()

###

# DOD:S weapons

dod = WeaponManager()

###

# TF2 weapons

tf = WeaponManager()

###

# Functions that point to the current game

def _getCurrentGame():
   gamename = str(es.ServerVar('eventscripts_gamedir')).replace('\\', '/').split('/')[~0]

   es.dbgmsg(1, 'Weaponlib: Current game - \"%s\"' % gamename)

   if gamename == 'cstrike':
      return cstrike
   elif gamename == 'hl2mp':
      return hl2mp
   elif gamename == 'dod':
      return dod
   elif gamename == 'tf':
      return tf

   raise NotImplementedError, 'Weaponlib does not support game \'%s\'' % gamename

currentgame = _getCurrentGame()


def getWeapon(name):
   return currentgame.getWeapon(name)
getWeapon.__doc__ = currentgame.getWeapon.__doc__


def getWeaponList(tag):
   return currentgame.getWeaponList(tag)
getWeaponList.__doc__ = currentgame.getWeaponList.__doc__


def getWeaponNameList(tag):
   return currentgame.getWeaponNameList(tag)
getWeaponNameList.__doc__ = currentgame.getWeaponNameList.__doc__


def getIndexList(tag):
   return currentgame.getIndexList(tag)
getIndexList.__doc__ = currentgame.getIndexList.__doc__